@@ -45,8 +45,7 @@ protected:
 
 	void TransferEnd(TransferEndReason reason);
 
-	bool InitBackend();
-	bool InitTls(const CTlsSocket* pPrimaryTlsSocket);
+	bool InitLayers(bool active);
 
 	void ResetSocket();
 
@@ -55,7 +54,7 @@ protected:
 	void OnAccept(int error);
 	void OnReceive();
 	void OnSend();
-	void OnClose(int error);
+	void OnSocketError(int error);
 	void OnTimer(fz::timer_id);
 
 	// Create a socket server
@@ -67,8 +66,6 @@ protected:
 	virtual void operator()(fz::event_base const& ev);
 	void OnIOThreadEvent();
 
-	std::unique_ptr<fz::socket> socket_{};
-
 	// Will be set only while creating active mode connections
 	std::unique_ptr<fz::listen_socket> socketServer_;
 
@@ -83,21 +80,17 @@ protected:
 	char *m_pTransferBuffer{};
 	int m_transferBufferLen{};
 
-	// Set to true if OnClose got called
-	// We now have to read all available data in the socket, ignoring any
-	// speed limits
-	bool m_onCloseCalled{};
-
 	bool m_postponedReceive{};
 	bool m_postponedSend{};
 	void TriggerPostponedEvents();
 
-	CBackend* m_pBackend{};
+	std::unique_ptr<fz::socket> socket_;
+	std::unique_ptr<CSocketBackend> ratelimit_layer_;
+	std::unique_ptr<CProxySocket> proxy_layer_;
+	std::unique_ptr<CTlsSocket> tls_layer_;
 
-	CProxySocket* m_pProxyBackend{};
+	SocketLayer* active_layer_{};
 
-	CTlsSocket* m_pTlsSocket{};
-	bool m_shutdown{};
 
 	// Needed for the madeProgress field in CTransferStatus
 	// Initially 0, 2 if made progress
