@@ -35,6 +35,7 @@ static const t_protocolInfo protocolInfos[] = {
 	{ DROPBOX,      L"dropbox",  true, 443, false, "Dropbox",                                                                 L"https" },
 	{ ONEDRIVE,     L"onedrive", true, 443, false, "Microsoft OneDrive",                                                      L"https" },
 	{ B2,           L"b2",       true, 443, false, "Backblaze B2",                                                            L"https" },
+	{ BOX,          L"box",      true, 443, false, "Box",                                                                     L"https" },
 	{ UNKNOWN,      L"",         false, 21, false, "", L"" }
 };
 
@@ -623,6 +624,15 @@ bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature
 			return true;
 		}
 		break;
+	case ProtocolFeature::TemporaryUrl:
+		if (protocol == S3 || protocol == DROPBOX || protocol == AZURE_BLOB || protocol == AZURE_FILE) {
+			return true;
+		}
+		break;
+	case ProtocolFeature::S3Sse:
+		if (protocol == S3) {
+			return true;
+		}
 	}
 	return false;
 }
@@ -685,6 +695,11 @@ void CServer::SetExtraParameter(std::string const& name, std::wstring const& val
 	}
 }
 
+void CServer::ClearExtraParameter(const std::string &name)
+{
+	extraParameters_.erase(name);
+}
+
 LogonType GetLogonTypeFromName(std::wstring const& name)
 {
 	if (name == _("Normal")) {
@@ -806,6 +821,7 @@ std::vector<LogonType> GetSupportedLogonTypes(ServerProtocol protocol)
 	case GOOGLE_DRIVE:
 	case DROPBOX:
 	case ONEDRIVE:
+	case BOX:
 		return {LogonType::interactive};
 	case HTTPS:
 	case UNKNOWN:
@@ -837,6 +853,17 @@ std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol pr
 			}();
 			return ret;
 		}
+	case S3:
+		{
+			static std::vector<ParameterTraits> ret = []() {
+				std::vector<ParameterTraits> ret;
+				ret.emplace_back(ParameterTraits{"ssealgorithm", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"ssekmskey", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"ssecustomerkey", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+				return ret;
+			}();
+			return ret;
+		}
 	default:
 		break;
 	}
@@ -865,6 +892,8 @@ std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
 		return std::tuple<std::wstring, std::wstring>{L"graph.microsoft.com", L""};
 	case B2:
 		return std::tuple<std::wstring, std::wstring>{L"api.backblazeb2.com", L""};
+	case BOX:
+		return std::tuple<std::wstring, std::wstring>{L"api.box.com", L""};
 	default:
 		break;
 	}
@@ -874,7 +903,7 @@ std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
 
 bool ProtocolHasUser(ServerProtocol protocol)
 {
-	return protocol != DROPBOX && protocol != ONEDRIVE;
+	return protocol != DROPBOX && protocol != ONEDRIVE && protocol != BOX;
 }
 
 bool CServer::SameResource(CServer const& other) const
