@@ -7,29 +7,35 @@ typedef std::make_signed_t<size_t> ssize_t;
 
 #include <gnutls/gnutls.h>
 #include "backend.h"
-#include "socket.h"
 
 #include <libfilezilla/buffer.hpp>
+#include <libfilezilla/socket.hpp>
 
 class CControlSocket;
 class CTlsSocket;
 class CTlsSocketImpl final
 {
 public:
-	CTlsSocketImpl(CTlsSocket& tlsSocket, fz::socket& pSocket, CControlSocket* pOwner);
+	CTlsSocketImpl(CTlsSocket& tlsSocket, CControlSocket* pOwner);
 	~CTlsSocketImpl();
 
-	int Handshake(const CTlsSocketImpl* pPrimarySocket = nullptr, bool try_resume = 0);
+	bool client_handshake(std::vector<uint8_t> const& session_to_resume, std::vector<uint8_t> const& required_certificate);
 
-	int Read(void *buffer, unsigned int size, int& error);
-	int Peek(void *buffer, unsigned int size, int& error);
-	int Write(const void *buffer, unsigned int size, int& error);
+	int connect(fz::native_string const& host, unsigned int port, fz::address_type family);
 
-	int Shutdown(bool silenceReadErrors);
+	int read(void *buffer, unsigned int size, int& error);
+	int write(void const* buffer, unsigned int size, int& error);
+
+	int shutdown();
 
 	void TrustCurrentCert(bool trusted);
 
-	CTlsSocket::TlsState GetState() const { return m_tlsState; }
+	fz::socket_state get_state() const {
+		return state_;
+	}
+
+	std::vector<uint8_t> get_session_parameters() const;
+	std::vector<uint8_t> get_raw_certificate() const;
 
 	std::wstring GetProtocolName();
 	std::wstring GetKeyExchange();
@@ -45,19 +51,16 @@ public:
 
 	static std::wstring GetGnutlsVersion();
 
-protected:
+private:
 	bool Init();
 	void Uninit();
 
 	bool InitSession();
 	void UninitSession();
-	bool CopySessionData(CTlsSocketImpl const* pPrimarySocket);
-
-	void OnRateAvailable(CRateLimiter::rate_direction direction);
-
+	
 	void ContinueWrite();
 	int ContinueHandshake();
-	void ContinueShutdown();
+	int ContinueShutdown();
 
 	int VerifyCertificate();
 	bool CertificateIsBlacklisted(std::vector<CCertificate> const& certificates);
@@ -70,15 +73,14 @@ protected:
 
 	static ssize_t PushFunction(gnutls_transport_ptr_t ptr, const void* data, size_t len);
 	static ssize_t PullFunction(gnutls_transport_ptr_t ptr, void* data, size_t len);
-	ssize_t PushFunction(const void* data, size_t len);
+	ssize_t PushFunction(void const* data, size_t len);
 	ssize_t PullFunction(void* data, size_t len);
 
 	int DoCallGnutlsRecordRecv(void* data, size_t len);
 
-	void TriggerEvents();
-
 	void operator()(fz::event_base const& ev);
 	void OnSocketEvent(fz::socket_event_source* source, fz::socket_event_flag t, int error);
+	void forward_hostaddress_event(fz::socket_event_source* source, std::string const& address);
 
 	void OnRead();
 	void OnSend();
@@ -90,9 +92,11 @@ protected:
 
 	void PrintVerificationError(int status);
 
+	void set_hostname(fz::native_string const& host);
+
 	CTlsSocket& tlsSocket_;
 
-	CTlsSocket::TlsState m_tlsState{ CTlsSocket::TlsState::noconn };
+	fz::socket_state state_{};
 
 	CControlSocket* m_pOwner{};
 
@@ -100,15 +104,13 @@ protected:
 	gnutls_session_t m_session{};
 
 	gnutls_certificate_credentials_t m_certCredentials{};
+	bool handshake_successful_{};
+	bool sent_closure_alert_{};
 
-	bool m_canReadFromSocket{true};
-	bool m_canWriteToSocket{true};
+	bool m_canReadFromSocket{false};
+	bool m_canWriteToSocket{false};
 
-	fz::socket& m_socket;
-	std::unique_ptr<CSocketBackend> socketBackend_;
-
-	bool shutdown_requested_{};
-	bool shutdown_silence_read_errors_{};
+	bool shutdown_silence_read_errors_{true};
 
 	// Due to the strange gnutls_record_send semantics, call it again
 	// with 0 data and 0 length after GNUTLS_E_AGAIN and store the number
@@ -116,13 +118,10 @@ protected:
 	// application.
 	// This avoids the rule to call it again with the -same- data after
 	// GNUTLS_E_AGAIN.
-	bool m_lastReadFailed{false};
 	bool m_lastWriteFailed{false};
 	unsigned int m_writeSkip{};
 
-	fz::buffer peekBuffer_;
-
-	gnutls_datum_t m_implicitTrustedCert{};
+	std::vector<uint8_t> required_certificate_;
 
 	bool m_socket_eof{};
 	int m_socket_error{ECONNABORTED}; // Set in the push and pull functions if reading/writing fails fatally
@@ -131,8 +130,6 @@ protected:
 	friend class CTlsSocketCallbacks;
 
 	fz::native_string hostname_;
-	unsigned int port_{};
-
 };
 
 #endif
