@@ -344,7 +344,7 @@ void CSiteManagerSite::SetControlVisibility(ServerProtocol protocol, LogonType t
 		return rows.end() - 1;
 	};
 
-	auto SetLabel = [](wxStaticText & label, ServerProtocol const protocol, std::string const& name) {
+	auto SetLabel = [](wxStaticText & label, ServerProtocol const, std::string const& name) {
 		if (name == "email") {
 			label.SetLabel(_("E-&mail account:"));
 		}
@@ -464,7 +464,7 @@ void CSiteManagerSite::SetLogonTypeCtrlState()
 {
 	LogonType const t = GetLogonType();
 	xrc_call(*this, "ID_USER", &wxTextCtrl::Enable, !predefined_ && t != LogonType::anonymous);
-	xrc_call(*this, "ID_PASS", &wxTextCtrl::Enable, !predefined_ && t == LogonType::normal || t == LogonType::account);
+	xrc_call(*this, "ID_PASS", &wxTextCtrl::Enable, !predefined_ && (t == LogonType::normal || t == LogonType::account));
 	xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::Enable, !predefined_ && t == LogonType::account);
 	xrc_call(*this, "ID_KEYFILE", &wxTextCtrl::Enable, !predefined_ && t == LogonType::key);
 	xrc_call(*this, "ID_KEYFILE_BROWSE", &wxButton::Enable, !predefined_ && t == LogonType::key);
@@ -526,28 +526,28 @@ bool CSiteManagerSite::Verify(bool predefined)
 	}
 
 	// Set selected type
-	ServerWithCredentials server;
-	server.SetLogonType(logon_type);
-	server.server.SetProtocol(protocol);
+	Site site;
+	site.SetLogonType(logon_type);
+	site.server.SetProtocol(protocol);
 
 	std::wstring port = xrc_call(*this, "ID_PORT", &wxTextCtrl::GetValue).ToStdWstring();
 	CServerPath path;
 	std::wstring error;
-	if (!server.ParseUrl(host, port, std::wstring(), std::wstring(), error, path, protocol)) {
+	if (!site.ParseUrl(host, port, std::wstring(), std::wstring(), error, path, protocol)) {
 		XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetFocus();
 		wxMessageBoxEx(error, _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 		return false;
 	}
 
-	XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(server.Format(ServerFormat::host_only));
-	if (server.server.GetPort() != CServer::GetDefaultPort(server.server.GetProtocol())) {
-		XRCCTRL(*this, "ID_PORT", wxTextCtrl)->ChangeValue(wxString::Format(_T("%d"), server.server.GetPort()));
+	XRCCTRL(*this, "ID_HOST", wxTextCtrl)->ChangeValue(site.Format(ServerFormat::host_only));
+	if (site.server.GetPort() != CServer::GetDefaultPort(site.server.GetProtocol())) {
+		XRCCTRL(*this, "ID_PORT", wxTextCtrl)->ChangeValue(wxString::Format(_T("%d"), site.server.GetPort()));
 	}
 	else {
 		XRCCTRL(*this, "ID_PORT", wxTextCtrl)->ChangeValue(wxString());
 	}
 
-	SetProtocol(server.server.GetProtocol());
+	SetProtocol(site.server.GetProtocol());
 
 	if (XRCCTRL(*this, "ID_CHARSET_CUSTOM", wxRadioButton)->GetValue()) {
 		if (XRCCTRL(*this, "ID_ENCODING", wxTextCtrl)->GetValue().empty()) {
@@ -598,7 +598,7 @@ bool CSiteManagerSite::Verify(bool predefined)
 	if (logon_type == LogonType::key) {
 		std::wstring keyFile = xrc_call(*this, "ID_KEYFILE", &wxTextCtrl::GetValue).ToStdWstring();
 		if (keyFile.empty()) {
-			wxMessageBox(_("You have to enter a key file path"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("You have to enter a key file path"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			xrc_call(*this, "ID_KEYFILE", &wxWindow::SetFocus);
 			return false;
 		}
@@ -623,7 +623,7 @@ bool CSiteManagerSite::Verify(bool predefined)
 		bool encrypted = !xrc_call(*this, "ID_PASS", &wxTextCtrl::GetHint).empty();
 		if (encrypted) {
 			if (pw.empty() != encryptionKey.empty()) {
-				wxMessageBox(_("You cannot change password and encryption key individually if using a master password."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("You cannot change password and encryption key individually if using a master password."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				xrc_call(*this, "ID_ENCRYPTIONKEY", &wxWindow::SetFocus);
 				return false;
 			}
@@ -632,7 +632,7 @@ bool CSiteManagerSite::Verify(bool predefined)
 		if (!encryptionKey.empty() || !encrypted) {
 			CStorjKeyInterface validator(this);
 			if (!validator.ValidateKey(encryptionKey, false)) {
-				wxMessageBox(_("You have to enter a valid encryption key"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("You have to enter a valid encryption key"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				xrc_call(*this, "ID_ENCRYPTIONKEY", &wxWindow::SetFocus);
 				return false;
 			}
@@ -688,7 +688,7 @@ void CSiteManagerSite::UpdateSite(Site &site)
 {
 	ServerProtocol const protocol = GetProtocol();
 	wxASSERT(protocol != UNKNOWN);
-	site.server_.server.SetProtocol(protocol);
+	site.server.SetProtocol(protocol);
 
 	unsigned long port;
 	if (!xrc_call(*this, "ID_PORT", &wxTextCtrl::GetValue).ToULong(&port) || !port || port > 65535) {
@@ -699,41 +699,41 @@ void CSiteManagerSite::UpdateSite(Site &site)
 	if (!host.empty() && host[0] == '[') {
 		host = host.substr(1, host.size() - 2);
 	}
-	site.server_.server.SetHost(host, port);
+	site.server.SetHost(host, port);
 
 	auto logon_type = GetLogonType();
-	site.server_.SetLogonType(logon_type);
+	site.SetLogonType(logon_type);
 
-	site.server_.SetUser(xrc_call(*this, "ID_USER", &wxTextCtrl::GetValue).ToStdWstring());
+	site.SetUser(xrc_call(*this, "ID_USER", &wxTextCtrl::GetValue).ToStdWstring());
 	auto pw = xrc_call(*this, "ID_PASS", &wxTextCtrl::GetValue).ToStdWstring();
 
-	if (protocol == STORJ && logon_type == LogonType::normal && (!pw.empty() || !site.server_.credentials.encrypted_)) {
+	if (protocol == STORJ && logon_type == LogonType::normal && (!pw.empty() || !site.credentials.encrypted_)) {
 		pw += '|';
 		pw += xrc_call(*this, "ID_ENCRYPTIONKEY", &wxTextCtrl::GetValue).ToStdWstring();
 	}
 
-	if (site.server_.credentials.encrypted_) {
+	if (site.credentials.encrypted_) {
 		if (!pw.empty()) {
-			site.server_.credentials.encrypted_ = fz::public_key();
-			site.server_.credentials.SetPass(pw);
+			site.credentials.encrypted_ = fz::public_key();
+			site.credentials.SetPass(pw);
 		}
 	}
 	else {
-		site.server_.credentials.SetPass(pw);
+		site.credentials.SetPass(pw);
 	}
-	site.server_.credentials.account_ = xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::GetValue).ToStdWstring();
+	site.credentials.account_ = xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::GetValue).ToStdWstring();
 
-	site.server_.credentials.keyFile_ = xrc_call(*this, "ID_KEYFILE", &wxTextCtrl::GetValue).ToStdWstring();
+	site.credentials.keyFile_ = xrc_call(*this, "ID_KEYFILE", &wxTextCtrl::GetValue).ToStdWstring();
 
-	site.m_comments = xrc_call(*this, "ID_COMMENTS", &wxTextCtrl::GetValue);
+	site.comments_ = xrc_call(*this, "ID_COMMENTS", &wxTextCtrl::GetValue).ToStdWstring();
 	site.m_colour = CSiteManager::GetColourFromIndex(xrc_call(*this, "ID_COLOR", &wxChoice::GetSelection));
 
 	std::wstring const serverType = xrc_call(*this, "ID_SERVERTYPE", &wxChoice::GetStringSelection).ToStdWstring();
-	site.server_.server.SetType(CServer::GetServerTypeFromName(serverType));
+	site.server.SetType(CServer::GetServerTypeFromName(serverType));
 
 	site.m_default_bookmark.m_localDir = xrc_call(*this, "ID_LOCALDIR", &wxTextCtrl::GetValue).ToStdWstring();
 	site.m_default_bookmark.m_remoteDir = CServerPath();
-	site.m_default_bookmark.m_remoteDir.SetType(site.server_.server.GetType());
+	site.m_default_bookmark.m_remoteDir.SetType(site.server.GetType());
 	site.m_default_bookmark.m_remoteDir.SetPath(xrc_call(*this, "ID_REMOTEDIR", &wxTextCtrl::GetValue).ToStdWstring());
 	site.m_default_bookmark.m_sync = xrc_call(*this, "ID_SYNC", &wxCheckBox::GetValue);
 	site.m_default_bookmark.m_comparison = xrc_call(*this, "ID_COMPARISON", &wxCheckBox::GetValue);
@@ -741,43 +741,43 @@ void CSiteManagerSite::UpdateSite(Site &site)
 	int hours = xrc_call(*this, "ID_TIMEZONE_HOURS", &wxSpinCtrl::GetValue);
 	int minutes = xrc_call(*this, "ID_TIMEZONE_MINUTES", &wxSpinCtrl::GetValue);
 
-	site.server_.server.SetTimezoneOffset(hours * 60 + minutes);
+	site.server.SetTimezoneOffset(hours * 60 + minutes);
 
 	if (xrc_call(*this, "ID_TRANSFERMODE_ACTIVE", &wxRadioButton::GetValue)) {
-		site.server_.server.SetPasvMode(MODE_ACTIVE);
+		site.server.SetPasvMode(MODE_ACTIVE);
 	}
 	else if (xrc_call(*this, "ID_TRANSFERMODE_PASSIVE", &wxRadioButton::GetValue)) {
-		site.server_.server.SetPasvMode(MODE_PASSIVE);
+		site.server.SetPasvMode(MODE_PASSIVE);
 	}
 	else {
-		site.server_.server.SetPasvMode(MODE_DEFAULT);
+		site.server.SetPasvMode(MODE_DEFAULT);
 	}
 
 	if (xrc_call(*this, "ID_LIMITMULTIPLE", &wxCheckBox::GetValue)) {
-		site.server_.server.MaximumMultipleConnections(xrc_call(*this, "ID_MAXMULTIPLE", &wxSpinCtrl::GetValue));
+		site.server.MaximumMultipleConnections(xrc_call(*this, "ID_MAXMULTIPLE", &wxSpinCtrl::GetValue));
 	}
 	else {
-		site.server_.server.MaximumMultipleConnections(0);
+		site.server.MaximumMultipleConnections(0);
 	}
 
 	if (xrc_call(*this, "ID_CHARSET_UTF8", &wxRadioButton::GetValue))
-		site.server_.server.SetEncodingType(ENCODING_UTF8);
+		site.server.SetEncodingType(ENCODING_UTF8);
 	else if (xrc_call(*this, "ID_CHARSET_CUSTOM", &wxRadioButton::GetValue)) {
 		std::wstring encoding = xrc_call(*this, "ID_ENCODING", &wxTextCtrl::GetValue).ToStdWstring();
-		site.server_.server.SetEncodingType(ENCODING_CUSTOM, encoding);
+		site.server.SetEncodingType(ENCODING_CUSTOM, encoding);
 	}
 	else {
-		site.server_.server.SetEncodingType(ENCODING_AUTO);
+		site.server.SetEncodingType(ENCODING_AUTO);
 	}
 
 	if (xrc_call(*this, "ID_BYPASSPROXY", &wxCheckBox::GetValue)) {
-		site.server_.server.SetBypassProxy(true);
+		site.server.SetBypassProxy(true);
 	}
 	else {
-		site.server_.server.SetBypassProxy(false);
+		site.server.SetBypassProxy(false);
 	}
 
-	UpdateExtraParameters(site.server_.server);
+	UpdateExtraParameters(site.server);
 }
 
 void CSiteManagerSite::UpdateExtraParameters(CServer & server)
@@ -863,30 +863,30 @@ void CSiteManagerSite::SetSite(Site const& site, bool predefined)
 		xrc_call(*this, "ID_ENCODING", &wxTextCtrl::Enable, false);
 	}
 	else {
-		xrc_call(*this, "ID_HOST", &wxTextCtrl::ChangeValue, site.server_.Format(ServerFormat::host_only));
-		unsigned int port = site.server_.server.GetPort();
+		xrc_call(*this, "ID_HOST", &wxTextCtrl::ChangeValue, site.Format(ServerFormat::host_only));
+		unsigned int port = site.server.GetPort();
 
-		if (port != CServer::GetDefaultPort(site.server_.server.GetProtocol())) {
+		if (port != CServer::GetDefaultPort(site.server.GetProtocol())) {
 			xrc_call(*this, "ID_PORT", &wxTextCtrl::ChangeValue, wxString::Format(_T("%d"), port));
 		}
 		else {
 			xrc_call(*this, "ID_PORT", &wxTextCtrl::ChangeValue, wxString());
 		}
 
-		ServerProtocol protocol = site.server_.server.GetProtocol();
+		ServerProtocol protocol = site.server.GetProtocol();
 		SetProtocol(protocol);
-		xrc_call(*this, "ID_BYPASSPROXY", &wxCheckBox::SetValue, site.server_.server.GetBypassProxy());
+		xrc_call(*this, "ID_BYPASSPROXY", &wxCheckBox::SetValue, site.server.GetBypassProxy());
 
-		LogonType const logonType = site.server_.credentials.logonType_;
+		LogonType const logonType = site.credentials.logonType_;
 		xrc_call(*this, "ID_LOGONTYPE", &wxChoice::SetStringSelection, GetNameFromLogonType(logonType));
 
 		SetControlVisibility(protocol, logonType);
 		SetLogonTypeCtrlState();
 
-		xrc_call(*this, "ID_USER", &wxTextCtrl::ChangeValue, site.server_.server.GetUser());
-		xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::ChangeValue, site.server_.credentials.account_);
+		xrc_call(*this, "ID_USER", &wxTextCtrl::ChangeValue, site.server.GetUser());
+		xrc_call(*this, "ID_ACCOUNT", &wxTextCtrl::ChangeValue, site.credentials.account_);
 
-		std::wstring pass = site.server_.credentials.GetPass();
+		std::wstring pass = site.credentials.GetPass();
 		std::wstring encryptionKey;
 		if (protocol == STORJ) {
 			size_t pos = pass.rfind('|');
@@ -896,7 +896,7 @@ void CSiteManagerSite::SetSite(Site const& site, bool predefined)
 			}
 		}
 
-		if (site.server_.credentials.encrypted_) {
+		if (site.credentials.encrypted_) {
 			xrc_call(*this, "ID_PASS", &wxTextCtrl::ChangeValue, wxString());
 			xrc_call(*this, "ID_ENCRYPTIONKEY", &wxTextCtrl::ChangeValue, wxString());
 
@@ -919,27 +919,27 @@ void CSiteManagerSite::SetSite(Site const& site, bool predefined)
 					continue;
 				}
 
-				it->second->ChangeValue(site.server_.credentials.GetExtraParameter(trait.name_));
+				it->second->ChangeValue(site.credentials.GetExtraParameter(trait.name_));
 				++it;
 			}
 		}
 
-		SetExtraParameters(site.server_.server);
+		SetExtraParameters(site.server);
 
-		xrc_call(*this, "ID_KEYFILE", &wxTextCtrl::ChangeValue, site.server_.credentials.keyFile_);
-		xrc_call(*this, "ID_COMMENTS", &wxTextCtrl::ChangeValue, site.m_comments);
+		xrc_call(*this, "ID_KEYFILE", &wxTextCtrl::ChangeValue, site.credentials.keyFile_);
+		xrc_call(*this, "ID_COMMENTS", &wxTextCtrl::ChangeValue, site.comments_);
 		xrc_call(*this, "ID_COLOR", &wxChoice::Select, CSiteManager::GetColourIndex(site.m_colour));
 
-		xrc_call(*this, "ID_SERVERTYPE", &wxChoice::SetSelection, site.server_.server.GetType());
+		xrc_call(*this, "ID_SERVERTYPE", &wxChoice::SetSelection, site.server.GetType());
 		xrc_call(*this, "ID_LOCALDIR", &wxTextCtrl::ChangeValue, site.m_default_bookmark.m_localDir);
 		xrc_call(*this, "ID_REMOTEDIR", &wxTextCtrl::ChangeValue, site.m_default_bookmark.m_remoteDir.GetPath());
 		xrc_call(*this, "ID_SYNC", &wxCheckBox::SetValue, site.m_default_bookmark.m_sync);
 		xrc_call(*this, "ID_COMPARISON", &wxCheckBox::SetValue, site.m_default_bookmark.m_comparison);
-		xrc_call<wxSpinCtrl, int>(*this, "ID_TIMEZONE_HOURS", &wxSpinCtrl::SetValue, site.server_.server.GetTimezoneOffset() / 60);
-		xrc_call<wxSpinCtrl, int>(*this, "ID_TIMEZONE_MINUTES", &wxSpinCtrl::SetValue, site.server_.server.GetTimezoneOffset() % 60);
+		xrc_call<wxSpinCtrl, int>(*this, "ID_TIMEZONE_HOURS", &wxSpinCtrl::SetValue, site.server.GetTimezoneOffset() / 60);
+		xrc_call<wxSpinCtrl, int>(*this, "ID_TIMEZONE_MINUTES", &wxSpinCtrl::SetValue, site.server.GetTimezoneOffset() % 60);
 
-		if (CServer::ProtocolHasFeature(site.server_.server.GetProtocol(), ProtocolFeature::TransferMode)) {
-			PasvMode pasvMode = site.server_.server.GetPasvMode();
+		if (CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::TransferMode)) {
+			PasvMode pasvMode = site.server.GetPasvMode();
 			if (pasvMode == MODE_ACTIVE) {
 				xrc_call(*this, "ID_TRANSFERMODE_ACTIVE", &wxRadioButton::SetValue, true);
 			}
@@ -951,7 +951,7 @@ void CSiteManagerSite::SetSite(Site const& site, bool predefined)
 			}
 		}
 
-		int const maxMultiple = site.server_.server.MaximumMultipleConnections();
+		int const maxMultiple = site.server.MaximumMultipleConnections();
 		xrc_call(*this, "ID_LIMITMULTIPLE", &wxCheckBox::SetValue, maxMultiple != 0);
 		if (maxMultiple != 0) {
 			xrc_call(*this, "ID_MAXMULTIPLE", &wxSpinCtrl::Enable, !predefined);
@@ -962,7 +962,7 @@ void CSiteManagerSite::SetSite(Site const& site, bool predefined)
 			xrc_call<wxSpinCtrl, int>(*this, "ID_MAXMULTIPLE", &wxSpinCtrl::SetValue, 1);
 		}
 
-		switch (site.server_.server.GetEncodingType()) {
+		switch (site.server.GetEncodingType()) {
 		default:
 		case ENCODING_AUTO:
 			xrc_call(*this, "ID_CHARSET_AUTO", &wxRadioButton::SetValue, true);
@@ -974,8 +974,8 @@ void CSiteManagerSite::SetSite(Site const& site, bool predefined)
 			xrc_call(*this, "ID_CHARSET_CUSTOM", &wxRadioButton::SetValue, true);
 			break;
 		}
-		xrc_call(*this, "ID_ENCODING", &wxTextCtrl::Enable, !predefined && site.server_.server.GetEncodingType() == ENCODING_CUSTOM);
-		xrc_call(*this, "ID_ENCODING", &wxTextCtrl::ChangeValue, site.server_.server.GetCustomEncoding());
+		xrc_call(*this, "ID_ENCODING", &wxTextCtrl::Enable, !predefined && site.server.GetEncodingType() == ENCODING_CUSTOM);
+		xrc_call(*this, "ID_ENCODING", &wxTextCtrl::ChangeValue, site.server.GetCustomEncoding());
 	}
 }
 
