@@ -85,7 +85,7 @@ CUpdater::CUpdater(CUpdateHandler& parent, CFileZillaEngineContext& engine_conte
 
 void CUpdater::Init()
 {
-	if (state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading) {
+	if (Busy()) {
 		return;
 	}
 
@@ -301,7 +301,7 @@ int CUpdater::Request(fz::uri const& uri)
 	pending_commands_.emplace_back(new CDisconnectCommand);
 
 	CServer server(fz::equal_insensitive_ascii(uri.scheme_, std::string("http")) ? HTTP : HTTPS, DEFAULT, fz::to_wstring_from_utf8(uri.host_), uri.port_);
-	pending_commands_.emplace_back(new CConnectCommand(server, Credentials()));
+	pending_commands_.emplace_back(new CConnectCommand(server, ServerHandle(), Credentials()));
 	pending_commands_.emplace_back(new CHttpRequestCommand(uri));
 
 	return ContinueDownload();
@@ -328,14 +328,14 @@ int CUpdater::ContinueDownload()
 
 bool CUpdater::CreateConnectCommand(std::wstring const& url)
 {
-	ServerWithCredentials s;
+	Site s;
 	CServerPath path;
 	std::wstring error;
 	if (!s.ParseUrl(url, 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
 		return false;
 	}
 
-	pending_commands_.emplace_back(new CConnectCommand(s.server, s.credentials));
+	pending_commands_.emplace_back(new CConnectCommand(s.server, s.Handle(), s.credentials));
 	return true;
 }
 
@@ -344,7 +344,7 @@ bool CUpdater::CreateTransferCommand(std::wstring const& url, std::wstring const
 	CFileTransferCommand::t_transferSettings transferSettings;
 	transferSettings.fsync = true;
 
-	ServerWithCredentials s;
+	Site s;
 	CServerPath path;
 	std::wstring error;
 	if (!s.ParseUrl(url, 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
@@ -925,4 +925,8 @@ bool CUpdater::UpdatableBuild() const
 	return CBuildInfo::GetBuildType() == _T("nightly") || CBuildInfo::GetBuildType() == _T("official");
 }
 
+bool CUpdater::Busy() const
+{
+	return state_ == UpdaterState::checking || state_ == UpdaterState::newversion_downloading;
+}
 #endif
