@@ -3,6 +3,8 @@
 
 #include "xmlfunctions.h"
 
+#include <set>
+
 class CertStore final
 {
 public:
@@ -11,6 +13,12 @@ public:
 	bool IsTrusted(CCertificateNotification const& notification);
 	void SetTrusted(CCertificateNotification const& notification, bool permanent, bool trustAllHostnames);
 
+	void SetInsecure(std::wstring const& host, unsigned int port, bool permanent);
+
+	bool IsInsecure(std::wstring const& host, unsigned int port, bool permanentOnly = false);
+
+	bool HasCertificate(std::wstring const& host, unsigned int port);
+
 private:
 	struct t_certData {
 		std::wstring host;
@@ -26,6 +34,8 @@ private:
 
 	std::list<t_certData> trustedCerts_;
 	std::list<t_certData> sessionTrustedCerts_;
+	std::set<std::tuple<std::wstring, unsigned int>> insecureHosts_;
+	std::set<std::tuple<std::wstring, unsigned int>> sessionInsecureHosts_;
 
 	CXmlFile m_xmlFile;
 };
@@ -44,10 +54,8 @@ private:
 
 	bool DisplayCert(wxDialogEx* pDlg, const CCertificate& cert);
 
-	void ParseDN(wxWindow* parent, const wxString& dn, wxSizer* pSizer);
-	void ParseDN_by_prefix(wxWindow* parent, std::list<wxString>& tokens, wxString prefix, const wxString& name, wxSizer* pSizer, bool decode = false);
-
-	wxString DecodeValue(const wxString& value);
+	void ParseDN(wxWindow* parent, std::wstring const& dn, wxSizer* pSizer);
+	void ParseDN_by_prefix(wxWindow* parent, std::vector<std::pair<std::wstring, std::wstring>>& tokens, std::wstring const& prefix, wxString const& name, wxSizer* pSizer);
 
 	std::vector<CCertificate> m_certificates;
 	wxDialogEx* m_pDlg{};
@@ -60,4 +68,6 @@ private:
 	CertStore & certStore_;
 };
 
+void ConfirmInsecureConection(CertStore & certStore, CInsecureFTPNotification & notification);
+
 #endif
